// drawWattieCan.cpp: A  program to display a texture mapped cylinder
// (c) 2005 Burkhard Wuensche, University of Auckland
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include <iostream>
#include <fstream>
using namespace std;
#include <math.h>
#include "Trackball.h"
#include "Lighting.h"
#include "Utilities.h"

const float Pi = 3.14159265358979323846264338327f;
const int windowWidth=400;
const int windowHeight=400;

#define NUM_SEGMENTS 16
#define NUM_SLICES 3

// material properties of the top and bottom of can
GLfloat mat_can_specular[4];
GLfloat mat_can_ambient_and_diffuse[4];
GLfloat mat_can_shininess[1];
// material properties of diffuse component of paper (colour given by texture)
GLfloat mat_paper_specular[4];
GLfloat mat_paper_ambient_and_diffuse[4];
GLfloat mat_paper_shininess[1];

// texture parameters
GLuint texName;
GLubyte *texture;
int textureWidth, textureHeight;

CTrackball trackball;
CLighting lighting;

typedef enum {NO_TEXTURE, UNSHADED, SHADED} RENDER_OPTION;
RENDER_OPTION current_render_option=UNSHADED;


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'n': current_render_option=NO_TEXTURE; 
					glutPostRedisplay(); break;
		case 'u': current_render_option=UNSHADED; 
					glutPostRedisplay(); break;
		case 's': current_render_option=SHADED; 
					glutPostRedisplay(); break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,6, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate the cylinder using the trackball ...

	float bottomY=-1, topY=1.5;
	float x,z,theta,s;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glShadeModel(GL_SMOOTH);

	if (current_render_option==UNSHADED)
	{
		lighting.disable();
		glBindTexture(GL_TEXTURE_2D, texName);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glEnable(GL_TEXTURE_2D);
		glBegin(GL_QUAD_STRIP);
		for (int segment=0; segment <= NUM_SEGMENTS; segment++)
		{
			s=(float) segment/(float) NUM_SEGMENTS;
			theta=2.0f*Pi*s;
			x = (float) cos(theta);		// x = r cos(theta), and r = 1
			z = (float) sin(theta);		// z = -r sin(theta)
			glTexCoord2f(1-s,0);
			glVertex3f(x,bottomY,z);
			glTexCoord2f(1-s,1);
			glVertex3f(x,topY,z);
		}
		glEnd();
		glDisable(GL_TEXTURE_2D);
		lighting.enable();
	}
	else
	{
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_paper_specular);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_paper_ambient_and_diffuse);
		glMaterialfv(GL_FRONT, GL_SHININESS, mat_paper_shininess);
		glBindTexture(GL_TEXTURE_2D, texName);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		if (current_render_option!=NO_TEXTURE) glEnable(GL_TEXTURE_2D);
		glBegin(GL_QUAD_STRIP);
		for (int segment=0; segment <= NUM_SEGMENTS; segment++)
		{
			s=(float) segment/(float) NUM_SEGMENTS;
			theta=2.0f*Pi*s;
			x = (float) cos(theta);		// x = r cos(theta), and r = 1
			z = (float) sin(theta);		// z = -r sin(theta)
			glNormal3f(x,0,z);
			glTexCoord2f(1-s,0);
			glVertex3f(x,bottomY,z);
			glTexCoord2f(1-s,1);
			glVertex3f(x,topY,z);
		}
		glEnd();
		glDisable(GL_TEXTURE_2D);
	}
	// top and bottom surface
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_can_specular);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_can_ambient_and_diffuse);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_can_shininess);
	int i,j;
	float t;
	for(i=0;i<NUM_SLICES;i++)
	{
		glBegin(GL_QUAD_STRIP);
		for(j=0;j<=NUM_SEGMENTS;j++)
		{	
			s=(float) i/(float) NUM_SLICES;
			t=2*Pi*(float) j/(float) NUM_SEGMENTS;
			glNormal3f(0,1,0);
			glVertex3f(s*cos(t),topY,s*sin(t));
			s=(float) (i+1)/(float) NUM_SLICES;
			glNormal3f(0,1,0);
			glVertex3f(s*cos(t),topY,s*sin(t));
		}	
		glEnd();
	}
	for(i=0;i<NUM_SLICES;i++)
	{
		glBegin(GL_QUAD_STRIP);
		for(j=0;j<=NUM_SEGMENTS;j++)
		{	
			s=(float) i/(float) NUM_SLICES;
			t=2*Pi*(float) j/(float) NUM_SEGMENTS;
			glNormal3f(0,-1,0);
			glVertex3f(s*cos(t),bottomY,s*sin(t));
			s=(float) (i+1)/(float) NUM_SLICES;
			glNormal3f(0,-1,0);
			glVertex3f(s*cos(t),bottomY,s*sin(t));
		}	
		glEnd();
	}

	glFlush ();
	glutSwapBuffers();

}


void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for black
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,8);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// material properties of top and bottom of can
	mat_can_ambient_and_diffuse[0]=0.53;		// silvery material ...
	mat_can_ambient_and_diffuse[1]=0.55;
	mat_can_ambient_and_diffuse[2]=0.57;
	mat_can_ambient_and_diffuse[3]=1;
	mat_can_specular[0]=0.2f;				// ... with white highlights
	mat_can_specular[1]=0.2f;				// if light source is reflected
	mat_can_specular[2]=0.2f;				// on the material surface.
	mat_can_specular[3]=1;
	mat_can_shininess[0]=50;
	mat_paper_ambient_and_diffuse[0]=1;		// modulate texture of printed paper 
	mat_paper_ambient_and_diffuse[1]=1;		// with illuminated white material
	mat_paper_ambient_and_diffuse[2]=1;
	mat_paper_ambient_and_diffuse[3]=1;
	mat_paper_specular[0]=0;			
	mat_paper_specular[1]=0;			
	mat_paper_specular[2]=0;			
	mat_paper_specular[3]=1;
	mat_paper_shininess[0]=0;

	// load texture
	ifstream textureFile;
	char* fileName="WattieCanTexture.ppm";
	textureFile.open(fileName, ios::in);
	if (textureFile.fail())
	{ 
		cout << "\n Error loading the texture"; cout.flush();
		exit(0);
	}
	skipLine(textureFile);
	skipLine(textureFile);
	textureFile >> textureWidth;
	textureFile >> textureHeight;
	int maxRGBValue;
	textureFile >> maxRGBValue;
	texture = new GLubyte[textureWidth*textureHeight*4];

	int k,l,c;
	for(k=textureHeight-1;k>=0;k--)
		for(l=0;l<textureWidth;l++)
		{
			textureFile >> c;
			texture[(k*textureWidth+l)*4]=(GLubyte) c;
			textureFile >> c;
			texture[(k*textureWidth+l)*4+1]=(GLubyte) c;
			textureFile >> c;
			texture[(k*textureWidth+l)*4+2]=(GLubyte) c;
			texture[(k*textureWidth+l)*4+3]=(GLubyte) 255;
		}

	// initialise wattie can texture
	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureWidth, textureHeight, 
				0, GL_RGBA, GL_UNSIGNED_BYTE, texture);	

}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Wattie Can");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
